# This code us copied unchanged from the unittest package that shipped
# with Python 2.7.4 and is distributed under the PSF License, version 2.
# Copyright the CPython developers and contributors.


class ErrorHolder(object):
    """
    Placeholder for a TestCase inside a result. As far as a TestResult
    is concerned, this looks exactly like a unit test. Used to insert
    arbitrary errors into a test suite run.
    """
    # Inspired by the ErrorHolder from Twisted:
    # http://twistedmatrix.com/trac/browser/trunk/twisted/trial/runner.py

    # attribute used by TestResult._exc_info_to_string
    failureException = None

    def __init__(self, description):
        self.description = description

    def id(self):
        return self.description

    def shortDescription(self):
        return None

    def __repr__(self):
        return "<ErrorHolder description=%r>" % (self.description,)

    def __str__(self):
        return self.id()

    def run(self, result):
        # could call result.addError(...) - but this test-like object
        # shouldn't be run anyway
        pass

    def __call__(self, result):
        return self.run(result)

    def countTestCases(self):
        return 0



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
from __future__ import absolute_import, unicode_literals

import logging

from .utils import get_module_by_name

logger = logging.getLogger(__name__)


class PluginError(Exception):

    pass


class PluginManager(object):

    def load_plugin_class(self, class_spec):
        if class_spec is None or '.' not in class_spec:
            msg = 'Malformed plugin factory specification {0!r}'.format(
                class_spec)
            logger.error(msg)
            raise PluginError(msg)
        module_name, factory_name = class_spec.rsplit('.', 1)
        try:
            module = get_module_by_name(module_name)
        except ImportError:
            msg = 'Unable to import {0!r}'.format(class_spec)
            logger.exception(msg)
            raise PluginError(msg)
        try:
            klass = getattr(module, factory_name)
        except AttributeError:
            msg = 'Module %r has no attribute {0!r}'.format(
                module.__name__, factory_name)
            logger.error(msg)
            raise PluginError(msg)
        if not callable(klass):
            msg = 'Plugin factory {0!r} is not callable'.format(klass)
            logger.error(msg)
            raise PluginError(msg)
        return klass

    def load_plugin(self, class_spec):
        if class_spec is None:
            return None
        klass = self.load_plugin_class(class_spec)
        return klass()



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
from __future__ import absolute_import, unicode_literals

from .testing import unittest
from .suite import TestSuite


class Loader(object):
    """Load individual test cases from modules and wrap them in the
    :class:`~haas.suite.Suite` container.

    """

    def __init__(self, test_suite_class=None, test_case_class=None,
                 test_method_prefix='test',
                 **kwargs):
        super(Loader, self).__init__(**kwargs)
        self._test_method_prefix = test_method_prefix

        if test_suite_class is None:
            test_suite_class = TestSuite
        self._test_suite_class = test_suite_class

        if test_case_class is None:
            test_case_class = unittest.TestCase
        self._test_case_class = test_case_class

    def create_suite(self, tests=()):
        """Create a test suite using the confugured test suite class.

        Parameters
        ----------
        tests : sequence
            Sequence of TestCase instances.

        """
        return self._test_suite_class(tests)

    def is_test_case(self, klass):
        """Check if a class is a TestCase.

        """
        return issubclass(klass, self._test_case_class)

    def find_test_method_names(self, testcase):
        """Return a list of test method names in the provided ``TestCase``
        subclass.

        Parameters
        ----------
        testcase : type
            Subclass of :class:`unittest.TestCase`

        """
        prefix = self._test_method_prefix
        names = [name for name in dir(testcase)
                 if name.startswith(prefix)
                 and hasattr(getattr(testcase, name), '__call__')]
        return names

    def load_test(self, testcase, method_name):
        """Create and return an instance of :class:`unittest.TestCase` for the
        specified unbound test method.

        Parameters
        ----------
        unbound_test : unbound method
            An unbound method of a :class:`unittest.TestCase`

        """
        if not self.is_test_case(testcase):
            raise TypeError(
                'Test case must be a subclass of '
                '{0.__module__}.{0.__name__}'.format(self._test_case_class))
        return testcase(methodName=method_name)

    def load_case(self, testcase):
        """Load a TestSuite containing all TestCase instances for all tests in
        a TestCase subclass.

        Parameters
        ----------
        testcase : type
            A subclass of :class:`unittest.TestCase`

        """
        tests = [self.load_test(testcase, name)
                 for name in self.find_test_method_names(testcase)]
        return self.create_suite(tests)

    def get_test_cases_from_module(self, module):
        """Return a list of TestCase subclasses contained in the provided
        module object.

        Parameters
        ----------
        module : module
            A module object containing ``TestCases``

        """
        module_items = (getattr(module, name) for name in dir(module))
        return [item for item in module_items
                if isinstance(item, type)
                and self.is_test_case(item)]

    def load_module(self, module):
        """Create and return a test suite containing all cases loaded from the
        provided module.

        Parameters
        ----------
        module : module
            A module object containing ``TestCases``

        """
        cases = self.get_test_cases_from_module(module)
        suites = [self.load_case(case) for case in cases]
        return self.create_suite(suites)



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
__all__ = [
    'unittest',
]

import sys


if sys.version_info[:2] == (2, 6):  # pragma: no cover
    import unittest2 as unittest
else:
    import unittest



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
from __future__ import absolute_import, unicode_literals

import time

from .testing import unittest


class TextTestResult(unittest.TextTestResult):
    """A simple extension to ``unittest.TextTestResult`` that displays
    progression of testing when run in verbose mode.

    """

    def __init__(self, total_tests, stream, descriptions, verbosity):
        """Create a TextTestResult. The parameters ``stream``, ``descriptions``
        and ``verbosity`` are as in ``unittest.TextTestResult``.

        Parameters
        ----------
        total_tests : int
            The total number of tests in the suite to be run, as
            returned by ``Suite.countTestCases()``

        """
        self._total_tests = total_tests
        super(TextTestResult, self).__init__(stream, descriptions, verbosity)

    def startTest(self, test):
        if self.showAll:
            padding = len(str(self._total_tests))
            prefix = '[{timestamp}] ({run: >{padding}d}/{total:d}) '.format(
                timestamp=time.ctime(),
                run=self.testsRun + 1,
                padding=padding,
                total=self._total_tests,
            )
            self.stream.write(prefix)
        super(TextTestResult, self).startTest(test)

    startTest.__doc__ = unittest.TextTestResult.startTest.__doc__
